/*
 * File : dataPacket.h
 *
 * Description : This file contains the declarations for 
 * the dataPacket class.
 */

#ifndef _dataPacket_h_
#define _dataPacket_h_

#include "list.h"

class dataPacket;

/*
 * Use macros to create type safe listMember and list
 * classes.
 */

DefineListMember(dataPacket);
DefineList(dataPacket);

/*
 * Class : dataPacket
 *
 * Description : The dataPacket class provides a 
 * implementation of a buffer. Functions to 
 * Append, Prepend, PreStrip and PostStrip are provided. 
 * The dataPacket keeps track of how many clients are 
 * using it. When the last client has deregistered, the
 * dataPacket deletes itself.
 *
 * Attributes:
 *	unsigned char *_buf : Pointer to allocated buffer.
 *	unsigned char *_head : The head of the circular buffer.
 *	unsigned char *_tail : The tail of the circular buffer.
 *	int _count : The number of users of this dataPacket.
 */

class dataPacket : public dataPacketMember {
 public:
	dataPacket();
	dataPacket(const unsigned char *data, int len);

	Boolean 	Prepend(const unsigned char *data, int len); 
	Boolean 	Append(const unsigned char *data, int len);
	Boolean		PreStrip(int len);
	Boolean		PostStrip(int len);

	int	Length() const { return (_tail - _head ); } 	
	const unsigned char *GetData() const { return _head; }

	void	Register();
	void	Deregister();
	void	Print();
 protected:
	unsigned char *_buf;
	unsigned char *_head;
	unsigned char *_tail;
	int	_count;

 private:
	~dataPacket();
	Boolean	InsertNewData(const unsigned char *data, int len);
};


#endif
