/*
 * File : layer.h
 *
 * Description : This file cotains the class layer declaration.
 */

#ifndef _layer_h_
#define _layer_h_

#include "types.h"

class dataPacket;

/*
 * Class : layer
 *
 * Description : The class layer is a base class for all layer 
 * classes. It provides a consistent interface to layers, as 
 * well as some flags to block transmission and reception.
 *
 * Attributes:
 *	Boolean _blockXmit : If True, transmission is blocked.
 *	Boolean _blockRecv : If True, reception is blocked.
 *	layer *_nPlus1 : Pointer to layer (N+1)
 *	layer *_nMinus1 : Pointer to layer (N-1)
 */

class layer {
 public:
	layer();
	virtual ~layer() {};

	virtual void Command(int cmd, int &errorCode);
	virtual void Command(int cmd, int &errorCode, int reason); 
	virtual void Command(int cmd, int &errorCode, dataPacket *data); 

	// Modifier functions to set private data
	void SetNPlus1Layer(layer *l)  { _nPlus1 = l; }
	void SetNMinus1Layer(layer *l) { _nMinus1 = l; }

	// functions to access private data
	layer *GetNPlus1Layer() { return _nPlus1; }
	layer *GetNMinus1Layer() { return _nMinus1; }

	void BlockXmit() { _blockXmit = TRUE; }
	void UnblockXmit() { _blockXmit = FALSE; }
	void BlockRecv() { _blockRecv = TRUE; }
	void UnblockRecv() { _blockRecv = FALSE; }

 protected:

	Boolean	_blockXmit;
	Boolean	_blockRecv;
	layer *	_nPlus1;
	layer *	_nMinus1;

};

#endif
